

import at.jku.ssw.PriorityQueue;
import at.jku.ssw.Iterator;
import at.jku.ssw.Out;

import at.jku.students.ArrayPriorityQueue;
import java.util.Comparator;

class PriorityQueueTest {

  public static void main(String[] args) {
    test1();
    test2();
    test3();
    test4();
    test5();
  }

  static void test1() {
    Out.println("*** Test 1 - offer/poll ***");
    PriorityQueue pq = createPriorityQueue();
    makeDot((ArrayPriorityQueue) pq, "test1.dot");
    
    boolean isFirst = true;
    Out.print("size()=" + pq.size() + ":");
    while (pq.size() > 0) {
      if (isFirst) {
        isFirst = false;
      } else {
        Out.print(",");
      }
      Out.print(" " + pq.poll());
    }
    Out.println();
  }

  static PriorityQueue createPriorityQueue() {
    return createPriorityQueue(
      new String[] { "x", "t", "o", "g", "s", "m", "n", "a", "e", "r", "i", "a", "o", "t", "o", "a" },
      String.CASE_INSENSITIVE_ORDER
    );    
  }
  
  static PriorityQueue createPriorityQueue(Object[] values, Comparator cmp) {
    PriorityQueue pq = new ArrayPriorityQueue(cmp);
    for (int i = 0; i < values.length; i++) {
      pq.offer(values[i]);
    }
    return pq;
  }
  
  static void makeDot(ArrayPriorityQueue priorityQueue, String filename) {
    Out.open(filename);
    Out.print(priorityQueue.makeDot());
    Out.close();
  }
  
  static void test2() {
    Out.println("\n*** Test 2 - iterator, contains ***");
    PriorityQueue pq = createPriorityQueue();
    printQueue(pq);
    for (char ch = 'a'; ch <= 'z'; ch++) {
      Out.println("contains(\"" + ch + "\")=" + pq.contains(Character.toString(ch)));
    }
  }
  
  static void printQueue(PriorityQueue queue) {
    Out.print("size=" + queue.size() + ": ");
    Iterator it = queue.iterator();
    boolean first = true;
    while (it.hasNext()) {
      if (!first) {
        Out.print(", ");
      } else {
        first = false;
      }
      Out.print(it.next());
    }
    Out.println();
  }

  private static class CharComparator implements Comparator<Character> {
    @Override
    public int compare(Character o1, Character o2) {
      return o1.compareTo(o2);
    }
  }

  static void test3() {
    Out.println("\n*** Test 3 - offer, poll, clear, peek ***");
    PriorityQueue pq = new ArrayPriorityQueue(new CharComparator());
    processCommand(pq, "Px,Pa,Ps,Py,Pd");
    makeDot((ArrayPriorityQueue) pq, "test3a.dot");
    printQueue(pq);
    processCommand(pq, "C,Pa,Pb,G,K,Pc,Pz,Pc,Pd");
    makeDot((ArrayPriorityQueue) pq, "test3b.dot");
    printQueue(pq);
    processCommand(pq, "Rc,Rz");
    makeDot((ArrayPriorityQueue) pq, "test3c.dot");
    printQueue(pq);
  }

  static void processCommand(PriorityQueue priorityQueue, String command) {
    for (String s : command.split(",")) {
      char ch;
      switch(s.charAt(0)) {
        case 'P':
          ch = s.charAt(1);
          priorityQueue.offer(ch);
          Out.println("offer(\"" + ch + "\")");
          break;
        case 'G':
          Out.println("poll()=" + priorityQueue.poll());
          break;
        case 'C':
          priorityQueue.clear();
          Out.println("clear()");
          break;
        case 'K':
          Out.println("peek()=" + priorityQueue.peek());
          break;
        case 'R':
          ch = s.charAt(1);
          Out.println("remove(\"" + ch + "\")=" + priorityQueue.remove(ch));
          break;
      }
    }
  }
  
  static void test4() {
    Out.println("\n*** Test 4 - Aufgabenstellung ***");
    PriorityQueue pq = createPriorityQueue(
      new Comparable[] { "t", "o", "g", "x", "s" }, String.CASE_INSENSITIVE_ORDER);
    makeDot((ArrayPriorityQueue) pq, "test4.dot");
    printQueue(pq);
  }
  
  static void test5() {
    Out.println("\n*** Test 5 - illegal arguments ***");
    PriorityQueue pq = new ArrayPriorityQueue(String.CASE_INSENSITIVE_ORDER);
    Out.print("offer(null): ");
    try {
      pq.offer(null);
    } catch (IllegalArgumentException ex) {
      Out.println(ex);
    }
    Out.print("contains(null): ");
    try {
      pq.contains(null);
    } catch (IllegalArgumentException ex) {
      Out.println(ex);
    }
    Out.print("remove(null): ");
    try {
      pq.remove(null);
    } catch (IllegalArgumentException ex) {
      Out.println(ex);
    }
  }
}